SystemOrganization addCategory: #'Seaside-Session'!
SystemOrganization addCategory: #'Seaside-Session-Filter'!

!WARequestContext methodsFor: '*seaside-session' stamp: 'jf 7/16/2008 10:49'!
application
	"Answer the current application or nil."

	^ handlers 
		detect: [ :each | each isApplication ]
		ifNone: [ nil ]! !

!WARequestContext methodsFor: '*seaside-session' stamp: 'lr 2/20/2009 11:05'!
redirectTo: aUrlOrString 
	self responseGenerator
		redirectTo: aUrlOrString;
		respond! !

!WARequestContext methodsFor: '*seaside-session' stamp: 'jf 7/25/2008 20:34'!
session
	"Answer the current session or nil."

	^ handlers 
		detect: [ :each | each isSession ]
		ifNone: [ nil ]! !

WAObject subclass: #WACurrentSession
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WACurrentSession commentStamp: 'jf 9/11/2008 14:58' prior: 0!
Available for backward compatibility. You should now use "self session" in any subclass of WAObject or "WACurrentRequestContext value session" anywhere else.!

!WACurrentSession class methodsFor: 'evaluating' stamp: 'jf 9/30/2009 01:02'!
value
	| context |
	self 
		greaseDeprecatedApi: 'WACurrentSession class>>#value'
		details: 'Use "self session" on a subclass of WAObject or "WACurrentRequestContext value session".'.
	[ context := WACurrentRequestContext value ] 
		on: WARequestContextNotFound
		do: [ :e | ^ nil ].
	^ context session! !

WAObject subclass: #WAMain
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WAMain commentStamp: 'rew 10/19/2007 10:49' prior: 0!
WAMain subclasses are used to initialize applications when a new session is started.

Subclasses must implement the following messages:
	start:
		Does the initializing 
!

!WAMain methodsFor: 'processing' stamp: 'jf 3/4/2009 12:25'!
start
	"The main entry point into a session. Subclasses should override this to do something
	and return a response."! !

!WAObject methodsFor: '*seaside-session' stamp: 'lr 7/25/2011 19:53'!
application
	"Answer the current seaside application, instance of WAApplication or a subclass."
	
	^ self requestContext application! !

!WAObject methodsFor: '*seaside-session' stamp: 'lr 7/25/2011 19:53'!
session
	"Answer the current seaside session, instance of WASesssion or a subclass."
	
	^ self requestContext session! !

WAObject subclass: #WASessionContinuation
	instanceVariableNames: 'states'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WASessionContinuation commentStamp: 'lr 6/2/2008 09:19' prior: 0!
I represent a continuation as part of the flow of pages within a session. I am not a real continuation (as compared to those used in #call: and #answer:), I only represent a specific point in the session. I reference the root component and a memory snapshot of backtracked objects.!

!WASessionContinuation methodsFor: 'evaluating' stamp: 'jf 1/21/2009 14:45'!
basicValue
	self withUnregisteredHandlerDo: [ self handleRequest ]! !

!WASessionContinuation methodsFor: 'evaluating' stamp: 'jf 1/21/2009 14:44'!
captureAndInvoke
	self captureState.
	self basicValue! !

!WASessionContinuation methodsFor: 'processing' stamp: 'jf 1/21/2009 14:11'!
captureState
	"self states reset"
	"Setting states to nil matches the behaviour of Seaside 2.8. Resetting the Snapshot
	is useful for JS callbacks in that refreshing the page will show the modified state
	but can also result in many ActionPhaseContinuations all sharing the same state
	but with different callbacks, some of which may reference state that is no longer present."
	states := nil.
	self updateStates: self states! !

!WASessionContinuation methodsFor: 'processing' stamp: 'jf 10/19/2008 12:35'!
handleRequest
	self subclassResponsibility! !

!WASessionContinuation methodsFor: 'private' stamp: 'lr 1/19/2009 19:09'!
redirectToContinuation: aSessionContinuation
	self respond: [ :response | response redirectTo: aSessionContinuation registerForUrl ]! !

!WASessionContinuation methodsFor: 'private' stamp: 'lr 3/8/2009 10:55'!
registerForUrl
	^ self registerForUrl: true! !

!WASessionContinuation methodsFor: 'private' stamp: 'lr 3/8/2009 10:55'!
registerForUrl: aBoolean
	| url |
	url := self session actionUrlForContinuation: self.
	aBoolean ifTrue: [ self updateUrl: url ].
	^ url! !

!WASessionContinuation methodsFor: 'accessing' stamp: 'lr 4/9/2008 10:52'!
request
	"Answer the current request object."

	^ self requestContext request! !

!WASessionContinuation methodsFor: 'processing' stamp: 'lr 5/20/2009 20:14'!
respond: aBlock 
	self requestContext respond: aBlock! !

!WASessionContinuation methodsFor: 'initialization' stamp: 'jf 10/16/2008 15:32'!
setStates: aSnapshot
	states := aSnapshot! !

!WASessionContinuation methodsFor: 'accessing' stamp: 'lr 1/19/2009 23:00'!
states
	^ states ifNil: [ states := WASnapshot new ]! !

!WASessionContinuation methodsFor: 'updating' stamp: 'jf 9/28/2008 23:07'!
updateStates: aSnapshot
	self session updateStates: aSnapshot! !

!WASessionContinuation methodsFor: 'updating' stamp: 'lr 9/8/2009 12:52'!
updateUrl: aUrl
	self session updateUrl: aUrl! !

!WASessionContinuation methodsFor: 'deprecated' stamp: 'jf 9/30/2009 01:02'!
url
	self 
		greaseDeprecatedApi: 'WASessionContinuation>>#url'
		details: 'Use WASessionContinuation>>registerForUrl.'.
	^ self registerForUrl! !

!WASessionContinuation methodsFor: 'evaluating' stamp: 'jf 1/21/2009 14:45'!
value
	"Resume processing of a request. To ensure valid application state restore all registered states."

	self states restore.
	self basicValue! !

!WASessionContinuation methodsFor: 'private' stamp: 'jf 11/3/2009 01:11'!
withUnregisteredHandlerDo: aBlock 
	^ aBlock 
		on: WAUnregisteredHandlerError
		do: [ :err | self requestContext redirectTo: self application url ]! !

!WARequestHandler methodsFor: '*seaside-session' stamp: 'mb 5/26/2006 08:49'!
isApplication
	^ false! !

!WARequestHandler methodsFor: '*seaside-session' stamp: 'jf 7/25/2008 20:33'!
isSession
	^ false! !

WARequestHandler subclass: #WASession
	instanceVariableNames: 'continuations properties jumpTo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WASession commentStamp: 'pmm 1/24/2009 10:24' prior: 0!
I am a Seaside session. A new instance of me gets created when an user accesses an application for the first time and is persistent as long as the user is interacting with it.

This class is intended to be subclasses by applications that need global state, like a user. Custom state can be added by creating instance variables and storing it there. The session can be retrieved by #session if inside a component or task or by evaluating: WACurrentRequestContext session

If the session has not been used for #defaultTimeoutSeconds, it is garbage collected by the system. To manually expire a session call #expire.

A good way to clear all sessions is the following code:

WARegistry clearAllHandlers.
WAPlatform current garbageCollect

Instance Variables
	cookiesEnabled:		<Boolean>
			
cookiesEnabled
	- Whether the client has a session cookie. This can only be true if #useSessionCookie is enabled.!

!WASession methodsFor: 'accessing' stamp: 'lr 4/15/2007 15:55'!
actionField
	^ '_k'! !

!WASession methodsFor: 'accessing' stamp: 'pmm 1/23/2009 21:02'!
actionUrlForContinuation: aContinuation
	^ self actionUrlForKey: (continuations store: aContinuation)! !

!WASession methodsFor: 'accessing' stamp: 'jf 2/15/2010 02:08'!
actionUrlForKey: aString
	^ self url
		addField: self actionField value: aString;
		yourself! !

!WASession methodsFor: 'deprecated' stamp: 'jf 9/30/2009 01:02'!
addLoadScript: aString 
	self 
		greaseDeprecatedApi: 'WASession>>#addLoadScript:'
		details: 'Use WAHtmlDocument>>#addLoadScript:. For example: "html document addLoadScript:".'.

	"We don't have access to the Canvas here so we can't provide an implementation"
	self error: 'You can no longer call #addLoadScript: on WASession.'! !

!WASession methodsFor: 'accessing' stamp: 'jf 9/27/2009 23:59'!
application
	^ self parent! !

!WASession methodsFor: 'deprecated' stamp: 'jf 11/3/2009 01:09'!
baseUrl
	self greaseDeprecatedApi: 'WASession>>#baseUrl'
		details: 'Use WAApplication>>#url.'.
	
	^ self application url! !

!WASession methodsFor: 'scripts' stamp: 'lr 4/15/2007 15:55'!
clearJumpTo
	jumpTo := nil! !

!WASession methodsFor: 'deprecated' stamp: 'jf 9/30/2009 01:02'!
closePopup
	self 
		greaseDeprecatedApi: 'WASession>>#closePopup'
		details: 'Use WARequestContext>>#closeThisPopupAndReloadOpener.'.
	^ self requestContext closeThisPopupAndReloadOpener! !

!WASession methodsFor: 'deprecated' stamp: 'jf 9/30/2009 01:02'!
closePopupWithoutReloadingOpener
	self 
		greaseDeprecatedApi: 'WASession>>#closePopupWithoutReloadingOpener'
		details: 'Use WARequestContext>>#closeThisPopup.'.
	^ self requestContext closeThisPopup! !

!WASession methodsFor: 'initialization' stamp: 'jf 1/10/2009 15:36'!
createCache
	"To configure the continuation cache you must currently subclass WASession and
	override this method."
	^ WACache new
		setExpiryPolicy: (WALeastRecentlyUsedExpiryPolicy new);
		yourself.! !

!WASession methodsFor: 'deprecated' stamp: 'jf 9/30/2009 01:02'!
currentRequest
	"Returns the current request object."
	self 
		greaseDeprecatedApi: 'WASession>>#currentRequest'
		details: 'Use WARequestContext>>#request directly. For example: "self requestContext request".'.
	^ self requestContext request! !

!WASession methodsFor: 'deprecated' stamp: 'jf 9/30/2009 01:02'!
expire
	self 
		greaseDeprecatedApi: 'WASession>>#expire'
		details: 'This method might be reimplemented again. In the meantime, if you just want to remove the Session from the Application, use WASession>>unregister (#unregistered will be called as a notification instead of #expired). Otherwise you should consider adding a Request Filter to the Session that implements whatever behaviour you want in order to block access to the Session.'.
	^ self unregister! !

!WASession methodsFor: 'handling' stamp: 'jf 2/22/2010 11:18'!
handleFiltered: aRequestContext
	| key continuation |
	key := aRequestContext request fields
		at: self actionField
		ifAbsent: [ ^ self start ].
		
	continuation := continuations
		at: key
		ifAbsent: [ ^ self unknownRequest ].
	continuation value! !

!WASession methodsFor: 'initialization' stamp: 'jf 1/29/2009 19:54'!
initialize
	super initialize.
	self initializeFilters.
	continuations := self createCache! !

!WASession methodsFor: 'initialization' stamp: 'jf 3/4/2009 10:13'!
initializeFilters
	self addFilter: WAMutualExclusionFilter new! !

!WASession methodsFor: 'testing' stamp: 'jf 7/25/2008 20:33'!
isSession
	^ true! !

!WASession methodsFor: 'scripts' stamp: 'lr 4/15/2007 15:55'!
jumpToAnchor: aString
	jumpTo := aString! !

!WASession methodsFor: 'accessing' stamp: 'jf 10/27/2008 14:25'!
key
	^ self application keyFor: self! !

!WASession methodsFor: 'deprecated' stamp: 'jf 11/3/2009 01:12'!
newSessionUrl
	self 
		greaseDeprecatedApi: 'WASession>>#newSessionUrl'
		details: 'Use WAApplication>>#url.'.
	^ self application url! !

!WASession methodsFor: 'copying' stamp: 'jf 11/3/2008 08:57'!
postCopy
	super postCopy.
	continuations := continuations copy! !

!WASession methodsFor: 'accessing' stamp: 'jf 9/30/2009 00:26'!
properties
	"Answer a dictionary of session-related properties."
	^ properties ifNil: [ properties := GRSmallDictionary new ]! !

!WASession methodsFor: 'deprecated' stamp: 'jf 9/30/2009 01:02'!
redirect
	self 
		greaseDeprecatedApi: 'WASession>>#redirect'
		details: 'Use WARequestContext>>#redirect (requires the Seaside-Flow package).'.
	^ self requestContext redirect! !

!WASession methodsFor: 'deprecated' stamp: 'jf 9/30/2009 01:02'!
redirectResponseFor: aUrlOrString 
	self 
		greaseDeprecatedApi: 'WASession>>#redirectResponseFor:'
		details: 'Use WARequestContext>>#redirectTo:.'.
	self requestContext redirectTo: aUrlOrString! !

!WASession methodsFor: 'deprecated' stamp: 'jf 9/30/2009 01:02'!
redirectTo: aUrlOrString 
	self 
		greaseDeprecatedApi: 'WASession>>#redirectTo:'
		details: 'Use WARequestContext>>#redirectTo:.'.
	self requestContext redirectTo: aUrlOrString! !

!WASession methodsFor: 'deprecated' stamp: 'jf 9/30/2009 01:02'!
redirectWithCookie: aCookie 
	self 
		greaseDeprecatedApi: 'WASession>>#redirectWithCookie:'
		details: 'Use "self requestContent response addCookie: aCookie. self requestContext redirect" (requires the Seaside-Flow package).'.
	self requestContext response addCookie: aCookie.
	self requestContext redirect! !

!WASession methodsFor: 'deprecated' stamp: 'jf 9/30/2009 01:02'!
redirectWithMessage: aString delay: aDuration 
	self 
		greaseDeprecatedApi: 'WASession>>#redirectWithMessage:delay:'
		details: 'Use WARequestContext>>#redirectWithMessage:delay: (requires the Seaside-Flow package).'.
	^ self requestContext 
		redirectWithMessage: aString
		delay: aDuration! !

!WASession methodsFor: 'deprecated' stamp: 'jf 5/28/2010 19:44'!
returnResponse: aResponse
	self 
		greaseDeprecatedApi: 'WASession>>#returnResponse:'
		details: 'Use WARequestContext>>#respond:. For example: "self requestContext respond: [:response | ... ]".'.

	"We can't really solve this consistenly."
	self error: 'You can no longer return a new WAResponse instance. You must instead modify the Response contained in the active RequestContext. Look for senders of #respond: and #response for examples.'! !

!WASession methodsFor: 'handling' stamp: 'jf 3/4/2009 09:50'!
start
	self application mainClass new start! !

!WASession methodsFor: 'handling' stamp: 'jf 10/19/2008 12:30'!
unknownRequest
	^ self start! !

!WASession methodsFor: 'updating' stamp: 'lr 9/8/2009 12:40'!
updateRoot: anHtmlRoot! !

!WASession methodsFor: 'updating' stamp: 'jf 3/4/2009 13:50'!
updateStates: aSnapshot
	self filters do: [ :each | each updateStates: aSnapshot ].
	aSnapshot register: filter! !

!WASession methodsFor: 'updating' stamp: 'lr 9/8/2009 12:50'!
updateUrl: aUrl! !

!WASession methodsFor: 'accessing' stamp: 'jf 11/3/2009 01:05'!
url
	| url |
	url := super url.
	jumpTo isNil
		ifFalse: [ url fragment: jumpTo ].
	^ url! !

WARequestFilter subclass: #WAMutualExclusionFilter
	instanceVariableNames: 'mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session-Filter'!

!WAMutualExclusionFilter methodsFor: 'processing' stamp: 'jf 2/5/2009 11:01'!
handleFiltered: aRequestContext
	(self shouldTerminate: aRequestContext)
		ifTrue: [ self terminate: aRequestContext ]
		ifFalse: [ mutex critical: [ super handleFiltered: aRequestContext ] ]! !

!WAMutualExclusionFilter methodsFor: 'initialization' stamp: 'jf 2/5/2009 10:54'!
initialize
	super initialize.
	mutex := WAMutex new! !

!WAMutualExclusionFilter methodsFor: 'private' stamp: 'jf 1/14/2009 13:34'!
shouldTerminate: aRequestContext
	^ (aRequestContext request fields includesKey: 'terminate')
		and: [ aRequestContext application preferenceAt: #sessionAllowTermination ]! !

!WAMutualExclusionFilter methodsFor: 'private' stamp: 'lr 5/20/2009 20:13'!
terminate: aRequestContext
	mutex terminateOwner.
	aRequestContext respond: [ :response |
		response
			contentType: WAMimeType textPlain;
			nextPutAll: 'Process terminated' ]! !

!WARequestFilter methodsFor: '*seaside-session' stamp: 'lr 8/31/2008 14:45'!
updateStates: aSnapshot
	aSnapshot register: next! !

!GRPackage class methodsFor: '*seaside-session' stamp: 'lr 2/17/2010 15:13'!
seasideSession
	^ self new
		name: 'Seaside-Session';
		description: 'Support for stateful and secure session handling.';
		addDependency: 'Seaside-Core';
		url: #seasideUrl;
		yourself! !

WASystemConfiguration subclass: #WAApplicationConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WAApplicationConfiguration commentStamp: 'jf 7/22/2008 12:40' prior: 0!
WAApplicationConfiguration defines attributes required by WAApplication. All applications should include this configuration (this is done by default for new applications).

See #addAttributes: for details on provided attributes.!

!WAApplicationConfiguration methodsFor: 'description' stamp: 'jf 11/3/2009 00:18'!
describeOn: config
	(config collection: #libraries)
		options: [ self libraryClasses ].
	(config list: #mainClass)
		beAdvanced;
		options: [ self mainClasses ];
		default: WAMain.
	(config string: #mimeType)
		beAdvanced;
		label: 'MIME Type';
		comment: 'The media type of HTML documents reported to the browser.';
		default: 'text/html'.
	(config url: #resourceBaseUrl)
		group: #server;
		comment: 'Sets the base URL for URLS generated with #resourceUrl:'.
	(config boolean: #sessionAllowTermination)
		comment: 'Whether sessions allows a special HTTP request to terminate the process currently running in the session. This is used during development and should be disabled for deployment.';
		default: false.
	(config list: #sessionClass)
		comment: 'Sessions in this application will be instances of this class.';
		options: [ self sessionClasses ];
		default: WASession
! !

!WAApplicationConfiguration methodsFor: 'description' stamp: 'pmm 3/14/2012 20:00'!
libraryClasses
	^ WAAbstractFileLibrary libraries asSortedCollection: [ :a :b | a name <= b name ]! !

!WAApplicationConfiguration methodsFor: 'description' stamp: 'jf 3/4/2009 12:22'!
mainClasses
	^ WAMain withAllSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !

!WAApplicationConfiguration methodsFor: 'ancestry' stamp: 'jf 1/30/2009 10:58'!
parents
	^ Array with: WARegistryConfiguration instance! !

!WAApplicationConfiguration methodsFor: 'description' stamp: 'jf 7/22/2008 12:00'!
sessionClasses
	^ WASession withAllSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !

WARegistry subclass: #WAApplication
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WAApplication commentStamp: 'rew 10/19/2007 10:47' prior: 0!
WAApplication is the starting point for a Seaside application. When a WAComponent is registered as a top level component a WAApplication object is added to a WADispatcher. The dispatcher forwards all requests to the WAApplication, which in turn forwards them to the correct WASession object. WAApplication's parent class WARegistry maintains a list of all active sessions to the application. 

"configuration" contains a chain of WAConfituration classes that define attributes of the application. The attribute "rootComponent", for example, defines the top level WAComponent class for the application. The configuration chain includes WAUserConfiguration, WAGlobalConfiguration, WARenderLoopConfiguration and WASessionConfiguration. Other configurations can be added to the chain when the top level application is registered with a dispatcher. (See below)

"libraries" is a collection of WALibrary classes, which are used to serve css, javascript and images used by the application. These may be in methods or in files. Sometimes these libraries are replaced by static files served by Apache. See WAFileLibrary class comment for more information.

Registering an Application.
	An application can be registered with a dispatcher by using the Seaside configuration page or via code. Below MyComponent is a subclass of WAComponent. The following registers the component as an application, gives some values to attributes (or preferences) and adds a library and a configuration. 

MyComponent class>>initialize
	"self initialize"
	| application |
	application := self registerAsApplication: 'sample'.
	application preferenceAt: #sessionClass put: Glorp.WAGlorpSession.
	application addLibrary: SampleLibrary.
	application configuration addAncestor: GlorpConfiguration new.
	application preferenceAt: #glorpDatabasePlatform put: Glorp.PostgreSQLPlatform.
	application preferenceAt: #databaseServer put: '127.0.0.1'.
	application preferenceAt: #databaseConnectString put: 'glorptests'.

MyComponent>>someInstanceMethod
	"example of how to access attributes (preferences)"
	self session application preferenceAt: #glorpDatabasePlatform!

!WAApplication class methodsFor: 'accessing' stamp: 'lr 11/16/2006 18:38'!
description
	^ 'Application'! !

!WAApplication methodsFor: 'libraries' stamp: 'lr 11/18/2007 15:24'!
addLibrary: aLibraryClass
	self preferenceAt: #libraries put: (self libraries copyWith: aLibraryClass)! !

!WAApplication methodsFor: 'configuration' stamp: 'pmm 5/26/2010 07:23'!
contentType
	^ self mimeType seasideMimeType
		charset: self requestContext charSet;
		yourself! !

!WAApplication methodsFor: 'configuration' stamp: 'jf 1/30/2009 17:25'!
defaultConfiguration
	^ super defaultConfiguration
		removeParent: WARegistryConfiguration instance;
		addParent: WAApplicationConfiguration instance;
		yourself! !

!WAApplication methodsFor: 'private' stamp: 'pmm 8/20/2010 06:40'!
doesHandlerSupportCookies: aRequestHandler
	^ aRequestHandler isSession! !

!WAApplication methodsFor: 'handling' stamp: 'jf 9/24/2009 00:12'!
handleDefault: aRequestContext
	self handle: aRequestContext registering: self newSession
		! !

!WAApplication methodsFor: 'handling' stamp: 'pmm 6/20/2011 21:01'!
handleFiltered: aRequestContext 
	"Seaside applications do not respond to prefetch requests and only understand GET and POST requests."
	
	aRequestContext request isPrefetch 
		ifTrue: [ self handlePrefetch: aRequestContext ]
		ifFalse: [
			(self isImplemented: aRequestContext request method)
				ifTrue: [ super handleFiltered: aRequestContext ]
				ifFalse: [
					aRequestContext responseGenerator
						notImplemented;
						respond ] ]! !

!WAApplication methodsFor: 'private' stamp: 'pmm 6/20/2011 21:13'!
handlePrefetch: aRequestContext
	"Mozilla prefetches some <link> elements:
	https://developer.mozilla.org/en/Link_prefetching_FAQ
	This usally isn't a problem as long as they are used for style sheets.
	It is however a problem when they are used for navigational 'Index', 'Next', and 'Prev'
	See the following bug:
	http://code.google.com/p/pier/issues/detail?id=132
	
	Disabling has the disadvantage that we also disable prefech of things that would support prefetching
	(like style sheets). However we don't recommend to have Seaside serving these in production.
	
	We are not actually sure whether 403 Forbidden is the correct way to indicate that we don't support prefetching.
	Ohter options would include:
		- 412 Precondition Failed, user agent has to try different headers (seems fitting, however seems to be also about entity header)
		- 405 Method Not Allowed, but this is about HTTP methods and we must send Allow containg the valid methods
		
	We don't go through the response generator because the user will never see the response
	so it doesn't make sense to customize the contents."
	aRequestContext respond: [ :response |
		response forbidden ]! !

!WAApplication methodsFor: 'testing' stamp: 'lr 7/25/2011 19:57'!
isApplication
	^ true! !

!WAApplication methodsFor: 'private' stamp: 'lr 1/12/2009 08:40'!
isImplemented: aString
	"Answer whether the receiver implements the given HTTP method."

	^ #('GET' 'POST') includes: aString! !

!WAApplication methodsFor: 'handling-helpers' stamp: 'jf 1/30/2009 13:10'!
keyField
	^ '_s'! !

!WAApplication methodsFor: 'configuration' stamp: 'lr 2/10/2008 15:59'!
libraries
	^ self preferenceAt: #libraries! !

!WAApplication methodsFor: 'configuration' stamp: 'jf 3/4/2009 09:50'!
mainClass
	^ self preferenceAt: #mainClass! !

!WAApplication methodsFor: 'configuration' stamp: 'pmm 4/6/2008 15:11'!
mimeType
	^ self preferenceAt: #mimeType! !

!WAApplication methodsFor: 'private' stamp: 'jf 9/24/2009 00:17'!
newSession
	"Create a new session which inherits our configuration."
	^ self sessionClass new
		configuration: (WAUserConfiguration new addParent: self configuration; yourself);
		yourself! !

!WAApplication methodsFor: 'libraries' stamp: 'lr 11/18/2007 15:25'!
removeLibrary: aLibraryClass
	self preferenceAt: #libraries put: (self libraries copyWithout: aLibraryClass)! !

!WAApplication methodsFor: 'configuration' stamp: 'lr 2/12/2007 22:02'!
resourceBaseUrl
	^ self preferenceAt: #resourceBaseUrl! !

!WAApplication methodsFor: 'configuration' stamp: 'jf 1/30/2009 17:21'!
sessionAllowTermination
	^ self preferenceAt: #sessionAllowTermination! !

!WAApplication methodsFor: 'configuration' stamp: 'lr 10/28/2007 14:42'!
sessionClass
	^ self preferenceAt: #sessionClass! !

!WAApplication methodsFor: 'accessing' stamp: 'lr 7/25/2011 19:03'!
sessions
	| collection |
	collection := OrderedCollection new.
	self sessionsDo: [ :session | collection add: session ].
	^ collection! !

!WAApplication methodsFor: 'enumerating' stamp: 'lr 7/25/2011 19:03'!
sessionsDo: aBlock
	self handlersDo: [ :handler | 
		handler isSession
			ifTrue: [ aBlock value: handler ] ]! !
